import { isFormGroup } from './helpers';
import { IFormFieldV1, IFormGroupV1, IFormV1 } from './models';
import { ICrossFieldConditionsParams, IFormFieldBase, IFormGroupBase } from './modelsCommon';
import { IFormSettings, IFormParsed, IFormSetting } from './modelsParsed';

const buildOptionsFromChildren = (
  children: (IFormFieldV1 | IFormGroupV1)[],
  formOptions = {} as IFormSettings,
): IFormSettings => {
  return children.reduce((acc, child) => {
    if (isFormGroup(child)) {
      return buildOptionsFromChildren(child.children, acc);
    }

    const fieldLayout = child.grid || {
      md: 12,
      sm: 12,
      xs: 12,
    };

    const crossFieldConditions: ICrossFieldConditionsParams = {};

    if (child.renderCondition) {
      crossFieldConditions.renderCondition = child.renderCondition;
    }

    if (child.disabledCondition) {
      crossFieldConditions.disabledCondition = child.disabledCondition;
    }

    const setting: IFormSetting = {
      fieldLayout,
    };

    if (crossFieldConditions?.disabledCondition || crossFieldConditions?.renderCondition) {
      setting.crossFieldConditions = crossFieldConditions;
    }

    acc[child.fieldName] = setting;

    return acc;
  }, formOptions);
};

const sanitizeChildrenFunction = (
  children: (IFormFieldV1 | IFormGroupV1)[],
): (IFormFieldBase | IFormGroupBase)[] => {
  return children.map((child) => {
    if (isFormGroup(child)) {
      return {
        ...child,
        children: sanitizeChildrenFunction(child.children),
      } as IFormGroupBase;
    }

    const { grid, renderCondition, disabledCondition, ...sanitizedChild } = child;
    return sanitizedChild;
  });
};

export const parseFormV1 = (form: IFormV1): IFormParsed => {
  const {
    submitButtonText,
    cancelButtonText,
    validationObject: { crossFieldValidations, ...sanitizedValidationObject },
    children,
  } = form;

  const formOptions = buildOptionsFromChildren(children);

  const fieldsWithCrossFieldValidations = Object.keys(crossFieldValidations);

  fieldsWithCrossFieldValidations.forEach((key) => {
    formOptions[key].crossFieldValidations = crossFieldValidations[key];
  });

  const sanitizedChildren = sanitizeChildrenFunction(children);

  const parsedForm: IFormParsed = {
    formatVersion: 'parsed',
    jsonSchema: sanitizedValidationObject,
    formSettings: formOptions,
    children: sanitizedChildren,
    submitButtonText,
  };

  if (cancelButtonText) {
    parsedForm.cancelButtonText = cancelButtonText;
  }

  return parsedForm;
};
